'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.runServiceHook = undefined;

var _sourceMapSupport2 = require('source-map-support');

/**
 * run service launch sequences
 */
let runServiceHook = exports.runServiceHook = (() => {
    var _ref = _asyncToGenerator(function* (launcher, hookName, ...args) {
        try {
            return yield Promise.all(launcher.map(function (service) {
                if (typeof service[hookName] === 'function') {
                    return service[hookName](...args);
                }
            }));
        } catch (e) {
            log.error(`A service failed in the '${hookName}' hook\n${e.stack}\n\nContinue...`);
        }
    });

    return function runServiceHook(_x, _x2) {
        return _ref.apply(this, arguments);
    };
})();

exports.getLauncher = getLauncher;

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

(0, _sourceMapSupport2.install)();


const log = (0, _wdioLogger2.default)('wdio-cli:utils');

/**
 * loads launch services
 */
function getLauncher(config) {
    let launchServices = [];

    if (!Array.isArray(config.services)) {
        return launchServices;
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = config.services[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            let serviceName = _step.value;

            let service;

            /**
             * allow custom services
             */
            if (typeof serviceName === 'object') {
                launchServices.push(serviceName);
                continue;
            }

            try {
                const pkgName = serviceName.startsWith('@') ? `${serviceName}/launcher` : `wdio-${serviceName}-service/launcher`;
                service = require(pkgName);
            } catch (e) {
                if (!e.message.match(`Cannot find module 'wdio-${serviceName}-service/launcher'`)) {
                    throw new Error(`Couldn't initialise launcher from service "${serviceName}".\n${e.stack}`);
                }
            }

            if (service && (typeof service.onPrepare === 'function' || typeof service.onComplete === 'function')) {
                launchServices.push(service);
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return launchServices;
}